@@ -2,4 +2,4 @@
 from citrine.citrine import Citrine  # noqa: F401
 import logging
 
-logging.basicConfig(level=logging.INFO)
+logging.basicConfig(level=logging.WARNING)
