@@ -3,11 +3,13 @@ from abc import abstractmethod
 import typing
 from typing import Optional
 from datetime import datetime
+from itertools import chain
 import uuid
 import arrow
 
-from taurus.entity.link_by_uid import LinkByUID
-from taurus.entity.dict_serializable import DictSerializable
+from gemd.enumeration.base_enumeration import BaseEnumeration
+from gemd.entity.link_by_uid import LinkByUID
+from gemd.entity.dict_serializable import DictSerializable
 from citrine._serialization.serializable import Serializable
 from citrine._serialization.polymorphic_serializable import PolymorphicSerializable
 
@@ -75,9 +77,19 @@ class Property(typing.Generic[DeserializedType, SerializedType]):
 
     def deserialize_from_dict(self, data: dict) -> DeserializedType:
         value = data
+        # `serialization_path` is expected to be a sequence of nested dictionary keys
         fields = self.serialization_path.split('.')
         for field in fields:
-            value = value.get(field, self.default)
+            next_value = value.get(field)
+            if next_value is None:
+                # This occurs if a `field` is unexpectedly not present in the data dictionary
+                # or if its value is null.
+                # Use the default value and stop traversing, even if we have not yet reached
+                # the last field in the serialization path.
+                value = self.default
+                break
+            else:
+                value = next_value
         base_class = _get_base_class(data, self.serialization_path)
         return self.deserialize(value, base_class=base_class)
 
@@ -324,7 +336,95 @@ class List(Property[list, list]):
         return serialized
 
 
-class MixedList(Property[list, list]):
+class Set(Property[set, typing.Iterable]):
+
+    def __init__(self,
+                 element_type: typing.Union[Property, typing.Type[Property]],
+                 serialization_path: typing.Optional[str] = None,
+                 serializable: bool = True,
+                 deserializable: bool = True,
+                 default: typing.Optional[DeserializedType] = None):
+        super().__init__(serialization_path,
+                         serializable,
+                         deserializable,
+                         default)
+        self.element_type = element_type if isinstance(element_type, Property) else element_type()
+
+    @property
+    def underlying_types(self):
+        return set
+
+    @property
+    def serialized_types(self):
+        return typing.Iterable
+
+    def _deserialize(self, value: typing.Iterable) -> set:
+        deserialized = set()
+        for element in value:
+            deserialized.add(self.element_type.deserialize(element))
+        return deserialized
+
+    def _serialize(self, value: set) -> set:
+        serialized = set()
+        for element in value:
+            serialized.add(self.element_type.serialize(element))
+        return serialized
+
+
+class Union(Property[typing.Any, typing.Any]):
+    """
+    One of several possible property types.
+
+    Attempted de/serialization is done in the order in which types are provided in the constructor.
+    """
+
+    def __init__(self,
+                 element_types: typing.Sequence[typing.Union[Property, typing.Type[Property]]],
+                 serialization_path: typing.Optional[str] = None,
+                 serializable: bool = True,
+                 deserializable: bool = True,
+                 default: typing.Optional[DeserializedType] = None):
+        super().__init__(serialization_path,
+                         serializable,
+                         deserializable,
+                         default)
+        if not isinstance(element_types, typing.Iterable):
+            raise ValueError("element types must be iterable: {}".format(element_types))
+        self.element_types: typing.List[Property, ...] = \
+            [el if isinstance(el, Property) else el() for el in element_types]
+
+    @property
+    def underlying_types(self):
+        all_underlying_types = [prop.underlying_types for prop in self.element_types]
+        return tuple(set(chain(*[typ if isinstance(typ, tuple)
+                                 else (typ,) for typ in all_underlying_types])))
+
+    @property
+    def serialized_types(self):
+        all_serialized_types = [prop.serialized_types for prop in self.element_types]
+        return tuple(set(chain(*[typ if isinstance(typ, tuple)
+                                 else (typ,) for typ in all_serialized_types])))
+
+    def _serialize(self, value: typing.Any) -> typing.Any:
+        for prop in self.element_types:
+            try:
+                return prop.serialize(value)
+            except ValueError:
+                pass
+        raise RuntimeError("An unexpected error occurred while trying to serialize {} to one "
+                           "of the following types: {}.".format(value, self.serialized_types))
+
+    def _deserialize(self, value: typing.Any) -> typing.Any:
+        for prop in self.element_types:
+            try:
+                return prop.deserialize(value)
+            except ValueError:
+                pass
+        raise RuntimeError("An unexpected error occurred while trying to deserialize {} to one "
+                           "of the following types: {}.".format(value, self.underlying_types))
+
+
+class SpecifiedMixedList(Property[list, list]):
     """A finite list in which the type of each entry is specified."""
 
     def __init__(self,
@@ -379,6 +479,35 @@ class MixedList(Property[list, list]):
         return serialized
 
 
+class Enumeration(Property[BaseEnumeration, str]):
+
+    def __init__(self,
+                 klass: typing.Type[typing.Any],
+                 serialization_path: typing.Optional[str] = None,
+                 serializable: bool = True,
+                 deserializable: bool = True,
+                 default: typing.Optional[DeserializedType] = None):
+        super().__init__(serialization_path,
+                         serializable,
+                         deserializable,
+                         default)
+        self.klass = klass
+
+    @property
+    def underlying_types(self):
+        return self.klass
+
+    @property
+    def serialized_types(self):
+        return str
+
+    def _deserialize(self, value: str) -> uuid.UUID:
+        return self.klass.get_enum(value)
+
+    def _serialize(self, value: typing.Any) -> str:
+        return self.klass.get_value(value)
+
+
 class Object(Property[typing.Any, dict]):
 
     def __init__(self,
@@ -425,9 +554,9 @@ class Object(Property[typing.Any, dict]):
             # There are two types of objects that we expect to not have fields.
             # One is a PolymorphicSerializable, in which case obj is some Serializable instance
             # of a child class, and has a dump() method.
-            # The other possibility is that obj is a taurus object that is not reproduced in
+            # The other possibility is that obj is a gemd object that is not reproduced in
             # citrine-python (attribute, value, bounds, etc.). These are all DictSerializable,
-            # and have a dump() method that uses the taurus json encoder client.
+            # and have a dump() method that uses the gemd json encoder client.
             try:
                 return obj.dump()
             except AttributeError:
@@ -458,7 +587,7 @@ class LinkOrElse(Property[typing.Union[Serializable, LinkByUID], dict]):
     (data concepts objects, templates, etc.) know how to deserialize themselves through a
     build() method that doesn't call _deserialize().
 
-    This is currently tied to LinkByUID in taurus, but could be modified to work with a
+    This is currently tied to LinkByUID in gemd, but could be modified to work with a
     generic Link object.
     """
 
