@@ -1,28 +1,25 @@
 """Top-level class for all data concepts objects and collections thereof."""
+from abc import abstractmethod, ABC
+from typing import TypeVar, Type, List, Union, Optional, Iterator, Iterable
 from uuid import UUID
-from typing import TypeVar, Type, List, Dict, Union, Optional, Iterator
-from copy import deepcopy
-from abc import abstractmethod
 
-from citrine._session import Session
 from citrine._rest.collection import Collection
+from citrine._serialization import properties
 from citrine._serialization.polymorphic_serializable import PolymorphicSerializable
+from citrine._serialization.properties import Property as SerializableProperty
 from citrine._serialization.serializable import Serializable
-from citrine._serialization.properties import Property, LinkOrElse, Object
-from citrine._serialization.properties import List as PropertyList
-from citrine._serialization.properties import Optional as PropertyOptional
-from citrine._utils.functions import (
-    validate_type, scrub_none,
-    replace_objects_with_links, get_object_id)
-from taurus.client.json_encoder import loads, dumps, LinkByUID
-from taurus.entity.dict_serializable import DictSerializable
-from taurus.entity.bounds.base_bounds import BaseBounds
-from taurus.entity.template.attribute_template import AttributeTemplate
-
+from citrine._session import Session
+from citrine._utils.functions import scrub_none, replace_objects_with_links
+from citrine.resources.audit_info import AuditInfo
 from citrine.resources.response import Response
+from gemd.entity.dict_serializable import DictSerializable
+from gemd.entity.link_by_uid import LinkByUID
+from gemd.json import GEMDJson
+
+CITRINE_SCOPE = 'id'
 
 
-class DataConcepts(PolymorphicSerializable['DataConcepts']):
+class DataConcepts(PolymorphicSerializable['DataConcepts'], DictSerializable, ABC):
     """
     An abstract data concepts object.
 
@@ -33,11 +30,6 @@ class DataConcepts(PolymorphicSerializable['DataConcepts']):
     typ: str
         A string denoting what type of DataConcepts class a particular instantiation is.
 
-    Attributes
-    ----------
-    session: Session
-        The Citrine session used to connect to the database.
-
     """
 
     _type_key = "type"
@@ -56,14 +48,76 @@ class DataConcepts(PolymorphicSerializable['DataConcepts']):
     Only populated if the :func:`get_type` method is invoked.
     """
 
+    json_support = None
+    """
+    Custom json support object, which knows how to serialize and deserialize DataConcepts classes.
+    """
+
+    client_specific_fields = {
+        "audit_info": AuditInfo,
+        "dataset": properties.UUID,
+    }
+    """
+    Fields that are added to the gemd data objects when they are used in this client
+
+    * AuditInfo contains who/when information about the resource on the citrine platform
+    """
+
     def __init__(self, typ: str):
         self.typ = typ
-        self.session = None
+        for field in self.client_specific_fields:
+            self.__setattr__("_{}".format(field), None)
+
+    @property
+    def audit_info(self) -> Optional[AuditInfo]:
+        """Get the audit info object."""
+        return self._audit_info
+
+    @property
+    def dataset(self) -> Optional[UUID]:
+        """[ALPHA] Get the dataset of this object, if it was returned by the backend."""
+        return self._dataset
 
     @classmethod
-    def build(cls, data: dict, session: Session = None):
+    def from_dict(cls, d: dict):
         """
-        Build a data concepts object from a dictionary or from a Taurus object.
+        Build a data concepts object from a dictionary.
+
+        This is an internal method, and should not be called directly by users.  First,
+        it removes client_specific_fields from d, if present, and then calls the gemd
+        object's from_dict method.  Finally, it adds those fields back.
+
+        Parameters
+        ----------
+        d: dict
+            A representation of the object that will be shallowly loaded into the object.
+
+        """
+        popped = {k: d.pop(k, None) for k in cls.client_specific_fields}
+        obj = super().from_dict(d)
+
+        for field, clazz in cls.client_specific_fields.items():
+            value = popped[field]
+            if value is None:
+                deserialized = None
+            elif issubclass(clazz, DictSerializable):
+                if not isinstance(value, dict):
+                    raise TypeError(
+                        "{} must be a dictionary or None but was {}".format(field, value))
+                deserialized = clazz.build(value)
+            elif issubclass(clazz, SerializableProperty):
+                # deserialize handles type checking already
+                deserialized = clazz(clazz).deserialize(value)
+            else:
+                raise NotImplementedError("No deserialization strategy reported for client "
+                                          "field type {} for field.".format(clazz, field))
+            setattr(obj, "_{}".format(field), deserialized)
+        return obj
+
+    @classmethod
+    def build(cls, data: dict):
+        """
+        Build a data concepts object from a dictionary or from a GEMD object.
 
         This is an internal method, and should not be called directly by users.
 
@@ -71,12 +125,10 @@ class DataConcepts(PolymorphicSerializable['DataConcepts']):
         ----------
         data: dict
             A representation of the object. It must be possible to put this dictionary through
-            the loads/dumps cycle of the Taurus
-            :py:mod:`JSON encoder <taurus.client.json_encoder>`. The ensuing dictionary must
+            the loads/dumps cycle of the GMED
+            :py:mod:`JSON encoder <gemd.jsonr>`. The ensuing dictionary must
             have a `type` field that corresponds to the response key of this class or of
-            :py:class:`LinkByUID <taurus.entity.link_by_uid.LinkByUID>`.
-        session: Session
-            the Citrine session to assign to the built object.
+            :py:class:`LinkByUID <gemd.entity.link_by_uid.LinkByUID>`.
 
         Returns
         -------
@@ -84,228 +136,7 @@ class DataConcepts(PolymorphicSerializable['DataConcepts']):
             An object corresponding to a data concepts resource.
 
         """
-        data_copy = deepcopy(data)
-        # Extract the values that are only for the client.
-        # They will be attached to the deserialized object later.
-        client_only_dict = dict()
-        for client_key in cls._client_keys:
-            val = DataConcepts._pop_field(data_copy, client_key)
-            client_only_dict[client_key] = val
-
-        # Running through a taurus loads/dumps cycle validates all of the fields and ensures
-        # the object is now a dictionary with a well-understood structure
-        # Note that this process modifies data_copy and makes it unsuitable for further use
-        data_copy_dict = loads(dumps(data_copy)).as_dict()
-
-        # Check the type--it should either correspond to LinkByUID or to this class.
-        if data_copy_dict.get(DataConcepts._type_key) == LinkByUID.typ:
-            return loads(dumps(data_copy_dict))
-        validate_type(data_copy_dict, cls._response_key)
-
-        # Remove the top-level "type" field and build child objects
-        data_copy_dict.pop(DataConcepts._type_key, None)
-        cls._build_child_objects(data_copy_dict, data)
-
-        data_concepts_object = cls(**data_copy_dict)
-        data_concepts_object.session = session
-        for client_key in cls._client_keys:
-            setattr(data_concepts_object, client_key, client_only_dict.get(client_key))
-
-        cls._build_discarded_objects(data_concepts_object, data, session)
-        return data_concepts_object
-
-    @staticmethod
-    def _get_field(data, field: str):
-        """
-        Get the value of a field from something that might be a dictionary or might be an object.
-
-        Parameters
-        ----------
-        data: dict or object
-            From which to get the value of the field
-        field: str
-            The name of the field
-
-        Returns
-        -------
-        Any
-            The value associated with `field` in `data`
-
-        """
-        if isinstance(data, dict):
-            return data.get(field, None)
-        elif isinstance(data, object):
-            return getattr(data, field, None)
-
-    @staticmethod
-    def _pop_field(data, field: str):
-        """
-        Pop the value of a field from something that might be a dictionary or might be an object.
-
-        Parameters
-        ----------
-        data: dict or object
-            From which to pop the value of the field
-        field: str
-            The name of the field
-
-        Returns
-        -------
-        Any
-            The value associated with `field` in `data`
-
-        """
-        if isinstance(data, dict):
-            return data.pop(field, None)
-        elif isinstance(data, object):
-            return data.__dict__.pop(field, None)
-
-    @classmethod
-    def _build_child_objects(cls, data: dict, data_with_soft_links,
-                             session: Session = None) -> dict:
-        """
-        Build the data concepts objects that this serialized object points to.
-
-        This method modifies the serialized object in place.
-
-        Parameters
-        ----------
-        data: dict
-            A data concepts object as a serialized dictionary.
-        data_with_soft_links: dict or \
-        :py:class:`DictSerializable <taurus.entity.dict_serializable.DictSerializable>`
-            A representation of data in which the knowledge of the soft-links is somehow encoded.
-        session: Session
-            Citrine session used to connect to the database.
-
-        Returns
-        -------
-        None
-            The data concepts object is modified so that all of its values or fields are
-            deserialized as DataConcepts objects.
-
-        """
-        def _is_dc(prop_type: Property) -> bool:
-            """Determine if a property is a DataConcepts object or LinkByUID."""
-            if isinstance(prop_type, LinkOrElse):
-                return True
-            elif isinstance(prop_type, Object):
-                return issubclass(prop_type.klass, DataConcepts)
-            elif isinstance(prop_type, PropertyOptional):
-                return _is_dc(prop_type.prop)
-            elif isinstance(prop_type, PropertyList):
-                return _is_dc(prop_type.element_type)
-            else:
-                return False
-
-        fields = {k: v for k, v in cls.__dict__.items() if isinstance(v, Property)}
-        for key, field in fields.items():
-            if _is_dc(field):
-                if data.get(key):
-                    if isinstance(data[key], List):
-                        data[key] = [DataConcepts.get_type(elem).build(elem) for
-                                     elem in DataConcepts._get_field(data_with_soft_links, key)]
-                        for elem in data[key]:
-                            if isinstance(elem, DataConcepts):
-                                elem.session = session
-                    else:
-                        elem = DataConcepts._get_field(data_with_soft_links, key)
-                        data[key] = DataConcepts.get_type(elem).build(elem)
-                        if isinstance(data[key], DataConcepts):
-                            data[key].session = session
-
-    @classmethod
-    def _build_discarded_objects(cls, obj, obj_with_soft_links, session: Session = None):
-        """
-        Possibly build objects that connect to obj but get removed during serialization.
-
-        This method is to be sparingly implemented, and the details will depend on the specific
-        soft-link.
-
-        This method modifies the object in place.
-
-        Parameters
-        ----------
-        obj: DataConcepts
-            A data concepts object that might be missing some soft links
-        obj_with_soft_links: dict or \
-        :py:class:`DictSerializable <taurus.entity.dict_serializable.DictSerializable>`
-            A representation of obj in which the knowledge of the soft-links is somehow encoded.
-        session: Session, optional
-            Citrine session used to connect to the database.
-
-        Returns
-        -------
-        None
-            The data concepts object is modified so that it has soft links to other objects.
-
-        """
-        pass
-
-    @staticmethod
-    def _build_list_of_soft_links(dc_obj, obj_with_soft_links, field: str, reverse_field: str,
-                                  linked_type, session: Session = None):
-        """
-        Build the data concepts objects that this object has soft links to.
-
-        This method is a specific implementation of _build_discarded_objects() that works
-        for one particular type of discarded object--a list of soft links.
-        In this case, object A has a field `field` such that A.field is a list [B1, B2, ...],
-        but the field is skipped upon serialization. We therefore cannot know about the links
-        if we serialize and then deserialize A. But if obj_with_soft_links retains the list,
-        then we can build each Bi and and set Bi.a = A to populate the soft link.
-
-        This method modifies the object in place.
-
-        Parameters
-        ----------
-        dc_obj: DataConcepts
-            A data concepts object that might be missing some soft links
-        obj_with_soft_links: dict or \
-        :py:class:`DictSerializable <taurus.entity.dict_serializable.DictSerializable>`
-            A representation of obj in which the knowledge of the soft-links is somehow encoded.
-        field: str
-            The name of the field that contains the list of soft links in obj.
-        reverse_field: str
-            The name of the field in the soft-linked objects that are used to refer back to obj.
-        linked_type: Type[DataConcepts]
-            The class of the soft-linked objects. Used for building them.
-        session: Session, optional
-            Citrine session used to connect to the database.
-
-        Returns
-        -------
-        None
-            The data concepts object, `dc_obj`, is modified so that all of its
-            soft-links are populated.
-
-        """
-        linked_objects = None
-        # Get the list of linked objects, if it exists.
-        if isinstance(obj_with_soft_links, dict):
-            if obj_with_soft_links.get(field):
-                linked_objects = obj_with_soft_links[field]
-        if isinstance(obj_with_soft_links, DictSerializable):
-            if hasattr(obj_with_soft_links, field):
-                linked_objects = getattr(obj_with_soft_links, field)
-        if linked_objects is None:
-            return
-
-        # Cycle through linked objects in obj_with_soft_link and if they are not LinkByUID,
-        # build them and then set their `reverse_field` field to dc_obj
-        for linked_obj in linked_objects:
-            assert isinstance(linked_obj, DictSerializable)
-            if isinstance(linked_obj, LinkByUID):
-                continue
-            # Sever the link between linked_obj and obj_with_soft_links.
-            # This prevents infinite loops in the next step.
-            setattr(linked_obj, reverse_field, None)
-            # Build a DataConcepts instance of linked_obj
-            dc_linked_obj = linked_type.build(linked_obj, session)
-            # Establish the link between the DataConcepts instances of obj and linked_obj
-            setattr(dc_linked_obj, reverse_field, dc_obj)
-            # Re-establish the link between linked_obj and obj_with_soft_links
-            setattr(linked_obj, reverse_field, obj_with_soft_links)
+        return cls.get_json_support().copy(data)
 
     @classmethod
     def get_type(cls, data) -> Type[Serializable]:
@@ -319,7 +150,7 @@ class DataConcepts(PolymorphicSerializable['DataConcepts']):
         ----------
         data: dict
             A dictionary corresponding to a serialized data concepts object of unknown type.
-            This method will also work if data is a deserialized Taurus object.
+            This method will also work if data is a deserialized GEMD object.
 
         Returns
         -------
@@ -328,7 +159,9 @@ class DataConcepts(PolymorphicSerializable['DataConcepts']):
 
         """
         if len(DataConcepts.class_dict) == 0:
-            DataConcepts._make_class_dict()
+            # This line is only reached if get_type is called before build,
+            # which is hard to reproduce, hence the no cover.
+            DataConcepts._make_class_dict()  # pragma: no cover
         if isinstance(data, DictSerializable):
             data = data.as_dict()
         return DataConcepts.class_dict[data['type']]
@@ -336,10 +169,6 @@ class DataConcepts(PolymorphicSerializable['DataConcepts']):
     @staticmethod
     def _make_class_dict():
         """Construct a dictionary from each type key to the class."""
-        from citrine.attributes.condition import Condition
-        from citrine.attributes.parameter import Parameter
-        from citrine.attributes.property import Property
-        from citrine.attributes.property_and_conditions import PropertyAndConditions
         from citrine.resources.condition_template import ConditionTemplate
         from citrine.resources.parameter_template import ParameterTemplate
         from citrine.resources.property_template import PropertyTemplate
@@ -354,8 +183,7 @@ class DataConcepts(PolymorphicSerializable['DataConcepts']):
         from citrine.resources.material_run import MaterialRun
         from citrine.resources.measurement_run import MeasurementRun
         from citrine.resources.process_run import ProcessRun
-        _clazz_list = [Condition, Parameter, Property, PropertyAndConditions,
-                       ConditionTemplate, ParameterTemplate, PropertyTemplate,
+        _clazz_list = [ConditionTemplate, ParameterTemplate, PropertyTemplate,
                        MaterialTemplate, MeasurementTemplate, ProcessTemplate,
                        IngredientSpec, MaterialSpec, MeasurementSpec, ProcessSpec,
                        IngredientRun, MaterialRun, MeasurementRun, ProcessRun]
@@ -363,15 +191,26 @@ class DataConcepts(PolymorphicSerializable['DataConcepts']):
             DataConcepts.class_dict[clazz._response_key] = clazz
         DataConcepts.class_dict['link_by_uid'] = LinkByUID
 
+    @classmethod
+    def get_json_support(cls):
+        """Get a DataConcepts-compatible json serializer/deserializer."""
+        if cls.json_support is None:
+            DataConcepts._make_class_dict()
+            cls.json_support = GEMDJson(scope=CITRINE_SCOPE)
+            cls.json_support.register_classes(
+                {k: v for k, v in DataConcepts.class_dict.items() if k != "link_by_uid"}
+            )
+        return cls.json_support
+
     def as_dict(self) -> dict:
-        """Dump to a dictionary (useful for interoperability with taurus)."""
+        """Dump to a dictionary (useful for interoperability with gemd)."""
         return self.dump()
 
 
 ResourceType = TypeVar('ResourceType', bound='DataConcepts')
 
 
-class DataConceptsCollection(Collection[ResourceType]):
+class DataConceptsCollection(Collection[ResourceType], ABC):
     """
     A collection of one kind of data concepts object.
 
@@ -412,17 +251,16 @@ class DataConceptsCollection(Collection[ResourceType]):
 
         Returns
         -------
-        DataConcepts
+        ResourceType
             A data model object built from the dictionary.
 
         """
         data_concepts_object = self.get_type().build(data)
-        data_concepts_object.session = self.session
         return data_concepts_object
 
-    def list(self, page: Optional[int] = None, per_page: Optional[int] = None):
+    def _fetch_page(self, page: Optional[int] = None, per_page: Optional[int] = None):
         """
-        List all visible elements of the collection.
+        List all visible elements of the collection.  Does not handle pagination.
 
         Parameters
         ----------
@@ -433,13 +271,59 @@ class DataConceptsCollection(Collection[ResourceType]):
 
         Returns
         -------
-        List[DataConcepts]
+        List[ResourceType]
             Every object in this collection.
 
         """
-        return self.filter_by_tags([], page, per_page)
+        return self.filter_by_tags([], page, per_page), ""
 
-    def register(self, model: ResourceType):
+    def _build_collection_elements(self,
+                                   collection: Iterable[dict]) -> Iterable[ResourceType]:
+        """
+        For each element in the collection, build the appropriate resource type.
+
+        Parameters
+        ---------
+        collection: Iterable[dict]
+            collection containing the elements to be built
+
+        Returns
+        -------
+        Iterable[ResourceType]
+            Resources in this collection.
+
+        """
+        return collection
+
+    def list(self,
+             page: Optional[int] = None,
+             per_page: Optional[int] = 100) -> List[ResourceType]:
+        """
+        List all visible elements of the collection.
+
+        Leaving page and per_page as default values will return a list of all elements
+        in the collection, paginating over all available pages.
+
+        Parameters
+        ---------
+        page: int, optional
+            The "page" of results to list. Default is to read all pages and return
+            all results.  This option is deprecated.
+        per_page: int, optional
+            Max number of results to return per page.  This parameter is used when
+            making requests to the backend service.  If the page parameter is
+            specified it limits the maximum number of elements in the response.
+
+        Returns
+        -------
+        List[ResourceType]
+            Every object in this collection.
+
+        """
+        # Convert the iterator to a list to avoid breaking existing client relying on lists
+        return list(super().list(page=page, per_page=per_page))
+
+    def register(self, model: ResourceType, dry_run=False):
         """
         Create a new element of the collection or update an existing element.
 
@@ -456,34 +340,80 @@ class DataConceptsCollection(Collection[ResourceType]):
 
         Parameters
         ----------
-        model: DataConcepts
+        model: ResourceType
             The DataConcepts object.
+        dry_run: bool
+            Whether to actually register the item or run a dry run of the register operation.
+            Dry run is intended to be used for validation. Default: false
 
         Returns
         -------
-        DataConcepts
+        ResourceType
             A copy of the registered object as it now exists in the database.
 
         """
         if self.dataset_id is None:
             raise RuntimeError("Must specify a dataset in order to register a data model object.")
         path = self._get_path()
+        params = {'dry_run': dry_run}
         # How do we prepare a citrine-python object to be the json in a POST request?
         # Right now, that method scrubs out None values and replaces top-level objects with links.
         # Eventually, we want to replace it with the following:
         #   dumped_data = dumps(loads(dumps(model.dump())))
         # This dumps the object to a dictionary (model.dump()), and then to a string (dumps()).
-        # But this string is still nested--because it's a dictionary, taurus.dumps() does not know
-        # how to replace the objects with link-by-uids. loads() converts this string into nested
-        # taurus objects, and then the final dumps() converts that to a json-ready string in which
-        # all of the object references have been replaced with link-by-uids.
+        # But this string is still nested--because it's a dictionary, GEMDJson.dumps() does not
+        # know how to replace the objects with link-by-uids. loads() converts this string into
+        # nested gemd objects, and then the final dumps() converts that to a json-ready string
+        # in which all of the object references have been replaced with link-by-uids.
+
+        GEMDJson(scope=CITRINE_SCOPE).dumps(model)  # This apparent no-op populates uids
         dumped_data = replace_objects_with_links(scrub_none(model.dump()))
-        data = self.session.post_resource(path, dumped_data)
+        data = self.session.post_resource(path, dumped_data, params=params)
         full_model = self.build(data)
-        model.session = self.session
         return full_model
 
-    def get(self, uid: Union[UUID, str], scope: str = 'id') -> ResourceType:
+    def register_all(self, models: List[ResourceType], dry_run=False) -> List[ResourceType]:
+        """
+        [ALPHA] Create or update each model in models.
+
+        This method has the same behavior as `register`, except that all no models will be
+        written if any one of them is invalid.
+
+        Using this method should yield significant improvements to write speed over separate
+        calls to `register`.
+
+        Parameters
+        ----------
+        models: List[ResourceType]
+            The objects to be written.
+        dry_run: bool
+            Whether to actually register the objects or run a dry run of the register operation.
+            Dry run is intended to be used for validation. Default: false
+
+        Returns
+        -------
+        List[ResourceType]
+            Each object model as it now exists in the database. The order and number of models
+            is guaranteed to be the same as originally specified.
+
+        """
+        if self.dataset_id is None:
+            raise RuntimeError("Must specify a dataset in order to register a data model object.")
+        path = self._get_path()
+        params = {'dry_run': dry_run}
+
+        json = GEMDJson(scope=CITRINE_SCOPE)
+        [json.dumps(x) for x in models]  # This apparent no-op populates uids
+
+        objects = [replace_objects_with_links(scrub_none(model.dump())) for model in models]
+        response_data = self.session.put_resource(
+            path + '/batch',
+            json={'objects': objects},
+            params=params
+        )
+        return [self.build(obj) for obj in response_data['objects']]
+
+    def get(self, uid: Union[UUID, str], scope: str = CITRINE_SCOPE) -> ResourceType:
         """
         Get the element of the collection with ID equal to uid.
 
@@ -492,11 +422,11 @@ class DataConceptsCollection(Collection[ResourceType]):
         uid: Union[UUID, str]
             The ID.
         scope: str
-            The scope of the uid, defaults to Citrine scope ('id')
+            The scope of the uid, defaults to Citrine scope (CITRINE_SCOPE)
 
         Returns
         -------
-        DataConcepts
+        ResourceType
             An object with specified scope and uid
 
         """
@@ -521,7 +451,7 @@ class DataConceptsCollection(Collection[ResourceType]):
 
         Returns
         -------
-        List[DataConcepts]
+        List[ResourceType]
             Every object in this collection that matches one of the tags.
             See (insert link) for a discussion of how to match on tags.
 
@@ -543,54 +473,6 @@ class DataConceptsCollection(Collection[ResourceType]):
             params=params)
         return [self.build(content) for content in response["contents"]]
 
-    def filter_by_attribute_bounds(
-            self,
-            attribute_bounds: Dict[Union[AttributeTemplate, LinkByUID], BaseBounds],
-            page: Optional[int] = None, per_page: Optional[int] = None):
-        """
-        Get all objects in the collection with attributes within certain bounds.
-
-        Currently only one attribute and one bounds on that attribute is supported.
-
-        Parameters
-        ----------
-        attribute_bounds: Dict[Union[AttributeTemplate, \
-        :py:class:`LinkByUID <taurus.entity.link_by_uid.LinkByUID>`], \
-        :py:class:`BaseBounds <taurus.entity.bounds.base_bounds.BaseBounds>`]
-            A dictionary from attributes to the bounds on that attribute.
-            Currently only real and integer bounds are supported.
-            Each attribute may be represented as an AttributeTemplate (PropertyTemplate,
-            ParameterTemplate, or ConditionTemplate) or as a LinkByUID,
-            but in either case there must be a uid and it must correspond to an
-            AttributeTemplate that exists in the database.
-            Only the uid is passed, so if you would like to update an attribute template you
-            must register that change to the database before you can use it to filter.
-        page: Optional[int]
-            The page of results to list, 1-indexed (i.e. the first page is page=1)
-        per_page: Optional[int]
-            The number of results to list per page
-
-        Returns
-        -------
-        List[DataConcepts]
-            List of all objects in this collection that both have the specified attribute
-            and have values within the specified bounds.
-
-        """
-        body = self._get_attribute_bounds_search_body(attribute_bounds)
-        params = {}
-        if self.dataset_id is not None:
-            params['dataset_id'] = str(self.dataset_id)
-        if page is not None:
-            params['page'] = page
-        if per_page is not None:
-            params['per_page'] = per_page
-
-        response = self.session.post_resource(
-            self._get_path(ignore_dataset=True) + "/filter-by-attribute-bounds",
-            json=body, params=params)
-        return [self.build(content) for content in response["contents"]]
-
     def filter_by_name(self, name: str, exact: bool = False,
                        page: Optional[int] = None, per_page: Optional[int] = None):
         """
@@ -610,7 +492,7 @@ class DataConceptsCollection(Collection[ResourceType]):
 
         Returns
         -------
-        List[DataConcepts]
+        List[ResourceType]
             List of every object in this collection whose `name` matches the search term.
 
         """
@@ -628,7 +510,7 @@ class DataConceptsCollection(Collection[ResourceType]):
         return [self.build(content) for content in response["contents"]]
 
     def list_by_name(self, name: str, exact: bool = False,
-                     forward: bool = True, per_page: int = 100) -> Iterator[DataConcepts]:
+                     forward: bool = True, per_page: int = 100) -> Iterator[ResourceType]:
         """
         Get all objects with specified name in this dataset.
 
@@ -648,7 +530,7 @@ class DataConceptsCollection(Collection[ResourceType]):
 
         Returns
         -------
-        Iterator[DataConcepts]
+        Iterator[ResourceType]
             List of every object in this collection whose `name` matches the search term.
 
         """
@@ -664,59 +546,7 @@ class DataConceptsCollection(Collection[ResourceType]):
             params=params)
         return (self.build(raw) for raw in raw_objects)
 
-    def list_by_attribute_bounds(
-            self,
-            attribute_bounds: Dict[Union[AttributeTemplate, LinkByUID], BaseBounds],
-            forward: bool = True, per_page: int = 100) -> Iterator[DataConcepts]:
-        """
-        Get all objects in the collection with attributes within certain bounds.
-
-        Results are ordered first by dataset, then by attribute value.
-
-        Currently only one attribute and one bounds on that attribute is supported, and
-        attribute type must be numeric.
-
-        Parameters
-        ----------
-        attribute_bounds: Dict[Union[AttributeTemplate, \
-        :py:class:`LinkByUID <taurus.entity.link_by_uid.LinkByUID>`], \
-        :py:class:`BaseBounds <taurus.entity.bounds.base_bounds.BaseBounds>`]
-            A dictionary from attributes to the bounds on that attribute.
-            Currently only real and integer bounds are supported.
-            Each attribute may be represented as an AttributeTemplate (PropertyTemplate,
-            ParameterTemplate, or ConditionTemplate) or as a LinkByUID,
-            but in either case there must be a uid and it must correspond to an
-            AttributeTemplate that exists in the database.
-            Only the uid is passed, so if you would like to update an attribute template you
-            must register that change to the database before you can use it to filter.
-        forward: bool
-            Set to False to reverse the order of results (i.e. return in descending order).
-        per_page: int
-            Controls the number of results fetched with each http request to the backend.
-            Typically, this is set to a sensible default and should not be modified. Consider
-            modifying this value only if you find this method is unacceptably latent.
-
-        Returns
-        -------
-        Iterator[DataConcepts]
-            List of every object in this collection whose `name` matches the search term.
-
-        """
-        body = self._get_attribute_bounds_search_body(attribute_bounds)
-        params = {}
-        if self.dataset_id is not None:
-            params['dataset_id'] = str(self.dataset_id)
-        raw_objects = self.session.cursor_paged_resource(
-            self.session.post_resource,
-            # "Ignoring" dataset because it is in the query params (and required)
-            self._get_path(ignore_dataset=True) + "/filter-by-attribute-bounds",
-            json=body,
-            forward=forward,
-            per_page=per_page,
-            params=params)
-        return (self.build(raw) for raw in raw_objects)
-
-    def list_all(self, forward: bool = True, per_page: int = 100) -> Iterator[DataConcepts]:
+    def list_all(self, forward: bool = True, per_page: int = 100) -> Iterator[ResourceType]:
         """
         Get all objects in the collection.
 
@@ -734,7 +564,7 @@ class DataConceptsCollection(Collection[ResourceType]):
 
         Returns
         -------
-        Iterator[DataConcepts]
+        Iterator[ResourceType]
             Every object in this collection.
 
         """
@@ -749,7 +579,7 @@ class DataConceptsCollection(Collection[ResourceType]):
             params=params)
         return (self.build(raw) for raw in raw_objects)
 
-    def list_by_tag(self, tag: str, per_page: int = 100) -> Iterator[DataConcepts]:
+    def list_by_tag(self, tag: str, per_page: int = 100) -> Iterator[ResourceType]:
         """
         Get all objects bearing a tag prefixed with `tag` in the collection.
 
@@ -771,11 +601,11 @@ class DataConceptsCollection(Collection[ResourceType]):
 
         Returns
         -------
-        Iterator[DataConcepts]
+        Iterator[ResourceType]
             Every object in this collection.
 
         """
-        params = {'tag': tag}
+        params = {'tags': [tag]}
         if self.dataset_id is not None:
             params['dataset_id'] = str(self.dataset_id)
         raw_objects = self.session.cursor_paged_resource(
@@ -785,7 +615,7 @@ class DataConceptsCollection(Collection[ResourceType]):
             params=params)
         return (self.build(raw) for raw in raw_objects)
 
-    def delete(self, uid: Union[UUID, str], scope: str = 'id'):
+    def delete(self, uid: Union[UUID, str], scope: str = CITRINE_SCOPE, dry_run: bool = False):
         """
         Delete the element of the collection with ID equal to uid.
 
@@ -794,25 +624,55 @@ class DataConceptsCollection(Collection[ResourceType]):
         uid: Union[UUID, str]
             The ID.
         scope: str
-            The scope of the uid, defaults to Citrine scope ('id')
+            The scope of the uid, defaults to Citrine scope (CITRINE_SCOPE)
+        dry_run: bool
+            Whether to actually delete the item or run a dry run of the delete operation.
+            Dry run is intended to be used for validation. Default: false
 
         """
         path = self._get_path() + "/{}/{}".format(scope, uid)
-        self.session.delete_resource(path)
+        params = {'dry_run': dry_run}
+        self.session.delete_resource(path, params=params)
         return Response(status_code=200)  # delete succeeded
 
-    @staticmethod
-    def _get_attribute_bounds_search_body(attribute_bounds):
-        if not isinstance(attribute_bounds, dict):
-            raise TypeError('attribute_bounds must be a dict mapping template to bounds; '
-                            'got {}'.format(attribute_bounds))
-        if len(attribute_bounds) != 1:
-            raise NotImplementedError('Currently, only searches with exactly one template '
-                                      'to bounds mapping are supported; got {}'
-                                      .format(attribute_bounds))
-        return {
-            'attribute_bounds': {
-                get_object_id(templ): bounds.as_dict()
-                for templ, bounds in attribute_bounds.items()
-            }
-        }
+    def _get_relation(self, relation: str, uid: Union[UUID, str], scope: str,
+                      forward: bool = True, per_page: int = 100) -> Iterator[ResourceType]:
+        """
+        Generic method for searching this collection by relation to another object.
+
+        Parameters
+        ----------
+        relation
+            Reflects the type of the object with the provided uid and scope, e.g.
+            'process-templates' if searching for process specs by process template.
+        uid
+            The unique ID of the object upon which this search is based, e.g. an
+            External or Citrine ID of a process template whose process spec usages
+            are being located.
+        scope
+            The scope of `uid`
+        forward
+            Whether to pages results in ascending order. Typically this is an
+            unnecessary parameter.
+        per_page
+            The number of results to retrieve in each request to the backend. Typically
+            this is an unnecessary parameter.
+        Returns
+        -------
+        Iterator[ResourceType]
+            Objects in this collection which are somehow related to the object with
+            provided uid and scope.
+
+        """
+        params = {}
+        if self.dataset_id is not None:
+            params['dataset_id'] = str(self.dataset_id)
+        raw_objects = self.session.cursor_paged_resource(
+            self.session.get_resource,
+            'projects/{}/{}/{}/{}/{}'.format(
+                self.project_id, relation, scope, uid, self._collection_key.replace('_', '-')),
+            forward=forward,
+            per_page=per_page,
+            params=params,
+            version='v1')
+        return (self.build(raw) for raw in raw_objects)
