@@ -10,7 +10,7 @@ CreationType = TypeVar('CreationType', bound=Predictor)
 
 
 class PredictorCollection(Collection[Predictor]):
-    """Represents the collection of all predictors for a project.
+    """[ALPHA] Represents the collection of all predictors for a project.
 
     Parameters
     ----------
@@ -22,6 +22,7 @@ class PredictorCollection(Collection[Predictor]):
     _path_template = '/projects/{project_id}/modules'
     _individual_key = None
     _resource = Predictor
+    _module_type = 'PREDICTOR'
 
     def __init__(self, project_id: UUID, session: Session):
         self.project_id = project_id
